#include "shaders/common/bge.hlsl"

Texture2D<float4> inputTex : register(t0);
RWTexture2D<float4> destTex : register(u0);

cbuffer ConstSettingsCBuffer : register(b0)
{
    // These are used in the gaussian blur of the
    // bright pass for the bloom effect.
    float gaussMultiplier;
    float gaussMean;
    float gaussStdDev;
};

cbuffer RuntimeSettingsCBuffer : register(b1)
{
    float2 oneOverTargetSize;
    float2 direction;
    uint2 destinationDimensions;
};

sampler samplerState : register(s0);

float computeGaussianValue( float x, float mean, float std_deviation )
{
    // The gaussian equation is defined as such:
    /*
        -(x - mean)^2
        -------------
        1.0               2*std_dev^2
        f(x,mean,std_dev) = -------------------- * e^
        sqrt(2*pi*std_dev^2)
    */

    float tmp = ( 1.0f / sqrt( 2.0f * PI * std_deviation * std_deviation ) );
    float tmp2 = exp( ( -( ( x - mean ) * ( x - mean ) ) ) / ( 2.0f * std_deviation * std_deviation ) );
    return tmp * tmp2;
}

// Define the number of threads to be executed in a single thread group when we dispatch the compute shader.
[numthreads(8, 8, 1)]
void main(uint3 dispatchThreadID : SV_DispatchThreadID)
{
    const uint2 storePos = uint2(dispatchThreadID.xy);

    float4 color = { 0.0f, 0.0f, 0.0f, 0.0f };
    float offsetX = 0.0f;
    float offsetY = 0.0f;
    float weight = 0.0f;
    float x = 0.0f;
    float fI = 0.0f;
	float sample = 63.0f;
	float sampleHalf = 0.0f;
	float blurSize = 4.0f;

    // Calculate uv position of current pixel.
    const float2 uv = (storePos + 0.5) * (1.0 / destinationDimensions.xy);
	sampleHalf = (sample-1)/2.0;
	
    for( int i = 0; i < sample; i++ )
    {
		
        fI = (float)i;
        offsetX = ((fI - sampleHalf) * oneOverTargetSize.x - oneOverTargetSize.x) * direction.x;
        offsetY = ((fI - sampleHalf) * oneOverTargetSize.y - oneOverTargetSize.y) * direction.y;
        x = (fI - sampleHalf);
        weight = computeGaussianValue( x, 0, 10.0 );
        color += (inputTex.SampleLevel(samplerState, uv + float2(offsetX, offsetY)*blurSize, 0) * weight);
    }
	

    destTex[storePos] = float4(color.rgb, 1.0f);
}