#include "shaders/common/gbuffer.h"

#include "shaders/common/hlsl.h"
#include "shaders/common/bge.hlsl"

#include "shaders/common/postfx/hdr/tonemap.h.hlsl"

Texture2D<float4> backbuffer : register(t0);

// luminanceTex
// size [1][1]
Texture2D<float4> adaptedLumTex : register(t1);

RWTexture2D<float4> destTex : register(u0);

cbuffer ConstSettingsCBuffer : register(b0)
{
    // The threshold luminace value for pixels which are
    // considered "bright" and need to be bloomed.
    float brightPassThreshold;  // brightPassThreshold * sunBrightFactor = 1.0f

    // The tone mapping middle grey or exposure value used
    // to adjust the overall "balance" of the image.
    //
    // 0.18 is fairly common value.
    float middleGray; // keyValue
};

cbuffer RuntimeSettingsCBuffer : register(b1)
{
    float2 oneOverTargetSize;
    uint2 destinationDimensions;
};

sampler samplerState : register(s0);

static const float3 LUMINANCE_VECTOR = float3(0.3125f, 0.6154f, 0.0721f);

static float2 gTapOffsets[4] =
{
   { -0.5, 0.5 },  { 0.5, -0.5 },
   { -0.5, -0.5 }, { 0.5, 0.5 }
};

// Define the number of threads to be executed in a single thread group when we dispatch the compute shader.
[numthreads(8, 8, 1)]
void main(uint3 dispatchThreadID : SV_DispatchThreadID)
{
    const uint2 storePos = uint2(dispatchThreadID.xy);

    float4 average = { 0.0f, 0.0f, 0.0f, 0.0f };

    // Calculate uv position of current pixel.
    const float2 uv = (storePos + 0.5) * (1.0 / destinationDimensions.xy);

    // Combine and average 4 samples from the source HDR texture.
    for( int i = 0; i < 4; i++ ) {
        average += (hdrDecode( backbuffer.SampleLevel(samplerState, uv + (gTapOffsets[i] * oneOverTargetSize), 0)));
    }
    average *= 0.25f;
	//average += hdrDecode( backbuffer.SampleLevel(samplerState, uv, 0));

    // Determine the brightness of this particular pixel.
    float adaptedLum = (adaptedLumTex.SampleLevel(samplerState, float2(0.5f, 0.5f), 0).r);
    float lum = (middleGray / (adaptedLum + 0.0001)) * hdrLuminance( average.rgb );
	const float newThresh = 3.0;
    //float lum = hdrLuminance( average.rgb );

    // Determine whether this pixel passes the test...
    if ( lum < newThresh ) {
        //average = float4( 0.0f, 0.0f, 0.0f, 1.0f );
		average *= saturate(pow(lum/newThresh, 16)); //soft thresh
    }

    // Write the colour to the bright-pass render target
    destTex[storePos] = hdrEncode(average);
}