Texture2D<float4> currentLum : register(t0);

// Last frame's adapted luminance
// size [1][1]
// GFXFormatR16F
Texture2D<float4> adaptedLumIn : register(t1);

// size [1][1]
// GFXFormatR16F
RWTexture2D<float4> adaptedLumOut : register(u0);

cbuffer ConstSettingsCBuffer : register(b0)
{
    // The rate of adaptation from the previous and new average scene luminance.
    float adaptRate;    // 2.0f
};

cbuffer RuntimeSettingsCBuffer : register(b1)
{
    float deltaRealTime;
};

// Define the number of threads to be executed in a single thread group when we dispatch the compute shader.
[numthreads(1, 1, 1)]
void main(uint3 dispatchThreadID : SV_DispatchThreadID)
{
    float fAdaptedLum = adaptedLumIn[uint2(0,0)].r;
    float fCurrentLum = currentLum[uint2(0,0)].r;

    // The user's adapted luminance level is simulated by closing the gap between
    // adapted luminance and current luminance by 2% every frame, based on a
    // 30 fps rate. This is not an accurate model of human adaptation, which can
    // take longer than half an hour.
    float diff = fCurrentLum - fAdaptedLum;
    float fNewAdaptation = fAdaptedLum + ( diff * ( 1.0 - exp( -deltaRealTime * 1.1f ) ) );

    // copy to luminance map
    adaptedLumOut[uint2(0, 0)] = float4(fNewAdaptation, 0, 0, 1);
}